# flake8: noqa
from service.stats.boot_time import Boot
from service.stats.cpu import Cpu
from service.stats.disk import Disk
from service.stats.memory import Memory
from service.stats.network import Network
from service.stats.system import System
from service.slack import Slack



from datetime import datetime
import argparse
from service import (
    Boot,
    Cpu,
    Disk,
    Memory,
    Network,
    System,
    Slack,
)


class ServiceCLI():
    def __init__(self):
        """Setup CLI args
        """
        parser = argparse.ArgumentParser(
            description='Service serves savvy server stats.'
        )
        parser.add_argument(
            '-b',
            '--boot',
            action='store_true',
            required=False,
            default=False,
            help='Show boot time stats.'
        )
        parser.add_argument(
            '-c',
            '--cpu',
            action='store_true',
            required=False,
            default=False,
            help='Show CPU stats.'
        )
        parser.add_argument(
            '-d',
            '--disk',
            action='store_true',
            required=False,
            default=False,
            help='Show disk stats.'
        )
        parser.add_argument(
            '-m',
            '--memory',
            action='store_true',
            required=False,
            default=False,
            help='Show memory stats.'
        )
        parser.add_argument(
            '-n',
            '--network',
            action='store_true',
            required=False,
            default=False,
            help='Show network stats.'
        )
        parser.add_argument(
            '-s',
            '--system',
            action='store_true',
            required=False,
            default=False,
            help='Show system stats.'
        )
        parser.add_argument(
            '-sl',
            '--slack',
            action='store_true',
            required=False,
            default=False,
            help='Send Service report to Slack.'
        )
        parser.parse_args(namespace=self)

    def run(self):
        Service.run(
            boot=self.boot,
            cpu=self.cpu,
            disk=self.disk,
            memory=self.memory,
            network=self.network,
            system=self.system,
            slack=self.slack,
        )


class Service():
    @classmethod
    def serve_data(cls, data_type):
        """Serve data from the specified category
        """
        data = data_type.serve()
        print(data)
        return data

    @classmethod
    def run(cls, boot=True, cpu=False, disk=False, memory=False,
            network=False, system=False, slack=False):
        """Run the Service app
        """
        # Preamble
        service_message = '='*15 + ' SERVICE ' + '=' * \
            15 + '\n' + f'Service Report ({datetime.now()})'
        print(service_message)

        boot_message = cls.serve_data(Boot) if boot else None
        cpu_message = cls.serve_data(Cpu) if cpu else None
        disk_message = cls.serve_data(Disk) if disk else None
        memory_message = cls.serve_data(Memory) if memory else None
        network_message = cls.serve_data(Network) if network else None
        system_message = cls.serve_data(System) if system else None

        final_message = (
            f'\n{service_message}'
            f'\n{boot_message}'
            f'\n{system_message}'
            f'\n{cpu_message}'
            f'\n{disk_message}'
            f'\n{memory_message}'
            f'\n{network_message}'
        )

        if slack:
            slack_output = Slack.message(final_message)
            print(slack_output)

        return final_message


def main():
    ServiceCLI().run()


if __name__ == '__main__':
    main()



import psutil
from service.stats.globals import Global


class Disk():
    @classmethod
    def serve(cls):
        """Serve disk info
        """
        # Title
        disk_title = '='*15 + ' Disk Information ' + '='*15
        partition_title = 'Partitions and Usage:'

        # Disk Information
        partitions = psutil.disk_partitions()
        disk = ''
        for partition in partitions:
            device = f'=== Device: {partition.device} ==='
            mountpoint = f'  Mountpoint: {partition.mountpoint}'
            filesystem_type = f'  File system type: {partition.fstype}'
            try:
                partition_usage = psutil.disk_usage(partition.mountpoint)
            except PermissionError:
                # Catch errors when a disk isn't ready
                continue
            total_size = f'  Total Size: {Global.get_size(partition_usage.total)}'  # noqa
            used = f'  Used: {Global.get_size(partition_usage.used)}'
            free = f'  Free: {Global.get_size(partition_usage.free)}'
            percentage = f'  Percentage: {partition_usage.percent}%'
            # Combine each disk into a variable
            disk += device + '\n' + mountpoint + '\n' + filesystem_type + \
                '\n' + total_size + '\n' + used + '\n' + free + '\n' + \
                percentage + '\n'

        # Get IO stats since boot
        disk_io = psutil.disk_io_counters()
        total_read = f'Total read (since boot): {Global.get_size(disk_io.read_bytes)}'  # noqa
        total_write = f'Total write (since boot): {Global.get_size(disk_io.write_bytes)}'  # noqa

        final_message = '\n' + disk_title + '\n' + partition_title + \
            '\n' + disk + '\n' + total_read + '\n' + total_write
        return final_message



import mock
from service.app import Service
from service.stats.boot_time import Boot


def test_serve_data():
    result = Service.serve_data(Boot)
    assert 'Boot Time' in result


@mock.patch('service.slack.Slack.message')
def test_app_slack(mock_slack):
    result = Service.run(boot=True, cpu=True, disk=True,
                         memory=True, network=True, system=True, slack=True)
    assert 'SERVICE' in result
    assert 'Boot Time' in result
    assert 'CPU Info' in result
    assert 'Disk Information' in result
    assert 'Memory Information' in result
    assert 'Network Information' in result
    assert 'System Information' in result
    mock_slack.assert_called_once()


@mock.patch('service.slack.Slack.message')
def test_app_no_slack(mock_slack):
    result = Service.run(boot=True, cpu=True, disk=True,
                         memory=True, network=True, system=True, slack=False)
    assert 'SERVICE' in result
    assert 'Boot Time' in result
    assert 'CPU Info' in result
    assert 'Disk Information' in result
    assert 'Memory Information' in result
    assert 'Network Information' in result
    assert 'System Information' in result
    mock_slack.assert_not_called()
